from sense.workflow.base.constants import Constants
from sense.workflow.base.utils import get_logger
from .sense_exceptions import SenseException
from ...client.address_api import AddressApi
from sense.workflow.provider.provider import Service

logger = get_logger()


class AddressService(Service):
    def __init__(self, *, client, label, name: str, pool: str, addr_type, scope=None, batch=None, netmask=None):
        super().__init__(label=label, name=name)
        self._client = client
        self.pool = pool
        self.addr_type = addr_type or 'IPv4'
        self.scope = scope or 'default'

        if self.addr_type == 'IPv4':
            self.netmask = netmask or '/32'
        else:
            self.netmask = netmask or '/128'

        self.batch = batch or 'subnet'
        self.address = str()
        self.hosts = list()

    def allocate_address(self):
        address_api = AddressApi(req_wrapper=self._client)
        response = address_api.allocate_address(self.pool, self.addr_type,
                                                self.name, scope=self.scope,
                                                batch=self.batch,
                                                netmask=self.netmask)
        try:
            if len(response) == 0 or "ERROR" in response:
                raise SenseException(f'Address allocate failed for {self.label}')

            from ipaddress import IPv4Network

            self.address = response

            if self.addr_type == 'IPv4':
                netmask = '/32'
            else:
                netmask = '/128'

            self.hosts = [str(host) + netmask for host in IPv4Network(response).hosts()]
        except ValueError as ve:
            raise SenseException(ve)

    # sense_util.py --address free,pool=AES-POOL,address='10.251.85.2/32'
    def free_address(self):
        address_api = AddressApi(req_wrapper=self._client)

        try:
            if self.address:
                response = address_api.free_address(self.pool, scope=self.scope, address=self.address)
            else:
                response = address_api.free_address(self.pool, scope=self.scope, name=self.name)

            if "ERROR" in response:
                raise SenseException(f'Address free failed with option {self.label}')
        except ValueError as ve:
            if "has no allocation matching" not in str(ve):
                raise SenseException(ve)


class AddressHandler:
    def __init__(self, *, provider):
        self.provider = provider

    @property
    def client(self):
        return self.provider.client

    @property
    def name(self):
        return self.provider.name

    @property
    def resource_listener(self):
        return self.provider.resource_listener

    @property
    def services(self):
        services = []

        for serv in self.provider.services:
            if isinstance(serv, AddressService):
                services.append(serv)

        return services

    def add_resource(self, *, resource: dict):
        assert Constants.POOL in resource
        assert resource[Constants.RES_COUNT] == 1
        label = resource[Constants.LABEL]
        pool = resource[Constants.POOL]
        addr_type = resource[Constants.ADDR_TYPE]
        assert addr_type in ['IPv4', 'IPv6']
        serv_name = self.provider.resource_name(resource, 0)
        netmask = resource.get(Constants.NET_MASK)
        serv = AddressService(label=label, name=serv_name, client=self.client, pool=pool,
                              netmask=netmask, addr_type=addr_type)
        self.provider.services.append(serv)
        self.resource_listener.on_added(source=self.provider, provider=self.provider, resource=serv)

    def create_resource(self, *, resource: dict):
        assert resource[Constants.RES_COUNT] == 1
        label = resource[Constants.LABEL]

        if self.provider.modified:
            for serv_name in set(self.provider.existing_map[label]) - set(self.provider.added_map[label]):
                serv_name = self.provider.resource_name(resource, 0)
                saved_state = next(filter(lambda s: s.attributes['name'] == serv_name,
                                          resource[Constants.SAVED_STATES]))
                pool = saved_state.attributes[Constants.POOL]
                addr_type = saved_state.attributes[Constants.ADDR_TYPE]
                netmask = saved_state.attributes.get(Constants.NET_MASK)
                serv = AddressService(label=label, name=serv_name, client=self.client, pool=pool,
                                      netmask=netmask, addr_type=addr_type)

                if 'address' in saved_state.attributes and saved_state.attributes['address']:
                    serv.address = saved_state.attributes['address']
                    serv.hosts = saved_state.attributes.get('hosts')
                    logger.info(f'Deleting resource {serv_name} using {serv.address}')
                    serv.free_address()

                self.resource_listener.on_deleted(source=self.provider, provider=self.provider, resource=serv)

        for serv in filter(lambda s: s.label == label, self.services):
            if serv.name in self.provider.existing_map[label]:
                saved_state = next(filter(lambda s: s.attributes['name'] == serv.name,
                                          resource[Constants.SAVED_STATES]), None)

                if saved_state:
                    serv.address = saved_state.attributes['address']
                    serv.hosts = saved_state.attributes['hosts']
                    logger.info(f"Resource: {serv.name} already exists:address={serv.address}")
                    continue

            if Constants.POOL not in resource:
                raise SenseException(f"should not happen. No pool ... {serv.name}")

            logger.info(f"Allocating resource: {serv.name}:{serv.address}")
            serv.allocate_address()

    def wait_for_create_resource(self, *, resource: dict):
        assert resource[Constants.RES_COUNT] == 1
        label = resource[Constants.LABEL]

        for serv in filter(lambda s: s.label == label, self.services):
            logger.debug(f'Resource {serv.name} has been created')
            self.resource_listener.on_created(source=self.provider, provider=self.provider, resource=serv)
            logger.info(f'Notified Resource {serv.name} has been created')

    def delete_resource(self, *, resource: dict):
        assert Constants.POOL in resource
        assert resource[Constants.RES_COUNT] == 1
        label = resource[Constants.LABEL]
        pool = resource[Constants.POOL]
        addr_type = resource[Constants.ADDR_TYPE]
        netmask = resource.get(Constants.NET_MASK)
        serv_name = self.provider.resource_name(resource, 0)
        serv = AddressService(label=label, name=serv_name, client=self.client, pool=pool,
                              netmask=netmask, addr_type=addr_type)
        saved_state = next(filter(lambda s: s.attributes['name'] == serv.name,
                                  resource[Constants.SAVED_STATES]), None)

        if saved_state:
            serv.address = saved_state.attributes['address']

        logger.info(f'Deleting resource {serv_name} using {serv.address}')
        serv.free_address()
        self.resource_listener.on_deleted(source=self.provider, provider=self.provider, resource=serv)



# coding: utf-8

# flake8: noqa
"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

# import models into model package
from sense.models.bandwidth import Bandwidth
from sense.models.capability_description import CapabilityDescription
from sense.models.connection import Connection
from sense.models.connection_bandwidth import ConnectionBandwidth
from sense.models.connection_path_tprofile import ConnectionPathTprofile
from sense.models.connection_schedule import ConnectionSchedule
from sense.models.connection_suggest_ip_range import ConnectionSuggestIpRange
from sense.models.connection_terminals import ConnectionTerminals
from sense.models.dnc_schema import DNCSchema
from sense.models.dnc_schema_data import DNCSchemaData
from sense.models.delta import Delta
from sense.models.discover_description import DiscoverDescription
from sense.models.discover_description_end_points import DiscoverDescriptionEndPoints
from sense.models.domain_description import DomainDescription
from sense.models.domain_description_edge_points import DomainDescriptionEdgePoints
from sense.models.full_profile import FullProfile
from sense.models.inline_response200 import InlineResponse200
from sense.models.intent import Intent
from sense.models.intent_expanded import IntentExpanded
from sense.models.log import Log
from sense.models.log_level import LogLevel
from sense.models.logging_configuration import LoggingConfiguration
from sense.models.logging_configuration_loggers import LoggingConfigurationLoggers
from sense.models.one_of_service_intent_data import OneOfServiceIntentData
from sense.models.operation import Operation
from sense.models.path import Path
from sense.models.profile_edit import ProfileEdit
from sense.models.profile_license import ProfileLicense
from sense.models.profile_manifest import ProfileManifest
from sense.models.service_description import ServiceDescription
from sense.models.service_intent import ServiceIntent
from sense.models.service_intent_queries import ServiceIntentQueries
from sense.models.service_intent_response import ServiceIntentResponse
from sense.models.slim_profile import SlimProfile
from sense.models.terminal import Terminal
from sense.models.verification_result import VerificationResult



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Intent(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'service_instance_uuid': 'str',
        'service_delta_uuid': 'str',
        'provisioned': 'bool',
        'json': 'str',
        'creation_time': 'str'
    }

    attribute_map = {
        'id': 'id',
        'service_instance_uuid': 'serviceInstanceUUID',
        'service_delta_uuid': 'serviceDeltaUUID',
        'provisioned': 'provisioned',
        'json': 'json',
        'creation_time': 'creation_time'
    }

    def __init__(self, id=None, service_instance_uuid=None, service_delta_uuid=None, provisioned=None, json=None, creation_time=None):  # noqa: E501
        """Intent - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._service_instance_uuid = None
        self._service_delta_uuid = None
        self._provisioned = None
        self._json = None
        self._creation_time = None
        self.discriminator = None
        self.id = id
        self.service_instance_uuid = service_instance_uuid
        self.service_delta_uuid = service_delta_uuid
        self.provisioned = provisioned
        self.json = json
        if creation_time is not None:
            self.creation_time = creation_time

    @property
    def id(self):
        """Gets the id of this Intent.  # noqa: E501

        Service intent UUID.  # noqa: E501

        :return: The id of this Intent.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Intent.

        Service intent UUID.  # noqa: E501

        :param id: The id of this Intent.  # noqa: E501
        :type: str
        """
        if id is None:
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def service_instance_uuid(self):
        """Gets the service_instance_uuid of this Intent.  # noqa: E501

        UUID for associated service instance.  # noqa: E501

        :return: The service_instance_uuid of this Intent.  # noqa: E501
        :rtype: str
        """
        return self._service_instance_uuid

    @service_instance_uuid.setter
    def service_instance_uuid(self, service_instance_uuid):
        """Sets the service_instance_uuid of this Intent.

        UUID for associated service instance.  # noqa: E501

        :param service_instance_uuid: The service_instance_uuid of this Intent.  # noqa: E501
        :type: str
        """
        if service_instance_uuid is None:
            raise ValueError("Invalid value for `service_instance_uuid`, must not be `None`")  # noqa: E501

        self._service_instance_uuid = service_instance_uuid

    @property
    def service_delta_uuid(self):
        """Gets the service_delta_uuid of this Intent.  # noqa: E501

        UUID for associated service delta.  # noqa: E501

        :return: The service_delta_uuid of this Intent.  # noqa: E501
        :rtype: str
        """
        return self._service_delta_uuid

    @service_delta_uuid.setter
    def service_delta_uuid(self, service_delta_uuid):
        """Sets the service_delta_uuid of this Intent.

        UUID for associated service delta.  # noqa: E501

        :param service_delta_uuid: The service_delta_uuid of this Intent.  # noqa: E501
        :type: str
        """
        if service_delta_uuid is None:
            raise ValueError("Invalid value for `service_delta_uuid`, must not be `None`")  # noqa: E501

        self._service_delta_uuid = service_delta_uuid

    @property
    def provisioned(self):
        """Gets the provisioned of this Intent.  # noqa: E501

        Whether or not the intent was or is successfully provisioned.  # noqa: E501

        :return: The provisioned of this Intent.  # noqa: E501
        :rtype: bool
        """
        return self._provisioned

    @provisioned.setter
    def provisioned(self, provisioned):
        """Sets the provisioned of this Intent.

        Whether or not the intent was or is successfully provisioned.  # noqa: E501

        :param provisioned: The provisioned of this Intent.  # noqa: E501
        :type: bool
        """
        if provisioned is None:
            raise ValueError("Invalid value for `provisioned`, must not be `None`")  # noqa: E501

        self._provisioned = provisioned

    @property
    def json(self):
        """Gets the json of this Intent.  # noqa: E501

        The service intent itself, in JSON string format.  # noqa: E501

        :return: The json of this Intent.  # noqa: E501
        :rtype: str
        """
        return self._json

    @json.setter
    def json(self, json):
        """Sets the json of this Intent.

        The service intent itself, in JSON string format.  # noqa: E501

        :param json: The json of this Intent.  # noqa: E501
        :type: str
        """
        if json is None:
            raise ValueError("Invalid value for `json`, must not be `None`")  # noqa: E501

        self._json = json

    @property
    def creation_time(self):
        """Gets the creation_time of this Intent.  # noqa: E501

        SQL Timestamp for intent creation.  # noqa: E501

        :return: The creation_time of this Intent.  # noqa: E501
        :rtype: str
        """
        return self._creation_time

    @creation_time.setter
    def creation_time(self, creation_time):
        """Sets the creation_time of this Intent.

        SQL Timestamp for intent creation.  # noqa: E501

        :param creation_time: The creation_time of this Intent.  # noqa: E501
        :type: str
        """

        self._creation_time = creation_time

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Intent, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Intent):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ServiceIntentResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'service_uuid': 'str',
        'intent_uuid': 'str',
        'queries': 'list[ServiceIntentQueries]',
        'model': 'str'
    }

    attribute_map = {
        'service_uuid': 'service_uuid',
        'intent_uuid': 'intent_uuid',
        'queries': 'queries',
        'model': 'model'
    }

    def __init__(self, service_uuid=None, intent_uuid=None, queries=None, model=None):  # noqa: E501
        """ServiceIntentResponse - a model defined in Swagger"""  # noqa: E501
        self._service_uuid = None
        self._intent_uuid = None
        self._queries = None
        self._model = None
        self.discriminator = None
        self.service_uuid = service_uuid
        self.intent_uuid = intent_uuid
        if queries is not None:
            self.queries = queries
        if model is not None:
            self.model = model

    @property
    def service_uuid(self):
        """Gets the service_uuid of this ServiceIntentResponse.  # noqa: E501


        :return: The service_uuid of this ServiceIntentResponse.  # noqa: E501
        :rtype: str
        """
        return self._service_uuid

    @service_uuid.setter
    def service_uuid(self, service_uuid):
        """Sets the service_uuid of this ServiceIntentResponse.


        :param service_uuid: The service_uuid of this ServiceIntentResponse.  # noqa: E501
        :type: str
        """
        if service_uuid is None:
            raise ValueError("Invalid value for `service_uuid`, must not be `None`")  # noqa: E501

        self._service_uuid = service_uuid

    @property
    def intent_uuid(self):
        """Gets the intent_uuid of this ServiceIntentResponse.  # noqa: E501


        :return: The intent_uuid of this ServiceIntentResponse.  # noqa: E501
        :rtype: str
        """
        return self._intent_uuid

    @intent_uuid.setter
    def intent_uuid(self, intent_uuid):
        """Sets the intent_uuid of this ServiceIntentResponse.


        :param intent_uuid: The intent_uuid of this ServiceIntentResponse.  # noqa: E501
        :type: str
        """
        if intent_uuid is None:
            raise ValueError("Invalid value for `intent_uuid`, must not be `None`")  # noqa: E501

        self._intent_uuid = intent_uuid

    @property
    def queries(self):
        """Gets the queries of this ServiceIntentResponse.  # noqa: E501


        :return: The queries of this ServiceIntentResponse.  # noqa: E501
        :rtype: list[ServiceIntentQueries]
        """
        return self._queries

    @queries.setter
    def queries(self, queries):
        """Sets the queries of this ServiceIntentResponse.


        :param queries: The queries of this ServiceIntentResponse.  # noqa: E501
        :type: list[ServiceIntentQueries]
        """

        self._queries = queries

    @property
    def model(self):
        """Gets the model of this ServiceIntentResponse.  # noqa: E501


        :return: The model of this ServiceIntentResponse.  # noqa: E501
        :rtype: str
        """
        return self._model

    @model.setter
    def model(self, model):
        """Sets the model of this ServiceIntentResponse.


        :param model: The model of this ServiceIntentResponse.  # noqa: E501
        :type: str
        """

        self._model = model

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ServiceIntentResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ServiceIntentResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



#!/usr/bin/env python3
"""
Title                   : siterm-api
Author                  : Justas Balcas
Email                   : jbalcas (at) es.net
@Copyright              : Copyright (C) 2024 ESnet
Date                    : 2024/06/05
"""
import os
import tempfile
from shutil import rmtree
import datetime
import json
import requests
import urllib3
from git import Repo
from sense.common import classwrapper
from sense.common import getConfig
from sense.common import loadYamlFile
from sense.common import getHTTPTimeout

@classwrapper
class GitRepo():
    """Git Repo Class for SENSE-SiteRM API to get all endpoints for sites"""
    def __init__(self, config):
        self.config = config

    def _getSiteRMRepo(self):
        """Get SiteRM Config Repo"""
        dirPath = tempfile.mkdtemp()
        Repo.clone_from(self.config['SITERM_REPO'], dirPath)
        return dirPath

    def getAllSiteRMs(self):
        """Get All SiteRM Configs"""
        fname = f"/tmp/siterm_endpoints_cache_{datetime.datetime.now().strftime('%Y-%m-%d')}.json"
        siteUrls = {}
        if os.path.isfile(fname):
            with open(fname, 'r', encoding="utf-8") as fd:
                return json.loads(fd.read())
        # If cache file not available, prepare a new one.
        repo = self._getSiteRMRepo()
        for dirName in os.listdir(repo):
            siteConfDir = os.path.join(repo, dirName)
            mappingFile = os.path.join(siteConfDir, 'mapping.yaml')
            if not os.path.isfile(mappingFile):
                continue
            mapping = loadYamlFile(mappingFile)
            for _key, val in mapping.items():
                if val.get('type', '') == 'FE' and val.get('config', ''):
                    tmpD = os.path.join(siteConfDir, val.get('config'))
                    confFile = os.path.join(tmpD, 'main.yaml')
                    if not os.path.isfile(confFile):
                        continue
                    conf = loadYamlFile(confFile)
                    webdomain = conf.get('general', {}).get('webdomain', '')
                    for site in conf.get('general', {}).get('sites', ''):
                        siteUrls.setdefault(site, webdomain)
        # Remove dir
        rmtree(repo, ignore_errors=True)
        # Save to cache file
        with open(fname, 'w', encoding="utf-8") as fd:
            fd.write(json.dumps(siteUrls))
        return siteUrls

@classwrapper
class RequestWrapper(GitRepo):
    """Request Wrapper for SENSE-SiteRM API"""
    def __init__(self):
        self.config = getConfig()
        self._setDefaults()
        self._validateConfig()
        self.cert = (self.config['SITERM_CERT'], self.config['SITERM_KEY'])
        if not self.config['SITERM_VERIFY']:
            urllib3.disable_warnings()
        super(RequestWrapper, self).__init__(self.config)
        self.fes = self.getAllSiteRMs()

    def _setDefaults(self):
        """Set Defaults for Config"""
        for key, val in {'SITERM_REPO': 'https://github.com/sdn-sense/rm-configs',
                         'SITERM_CERT': '/etc/grid-security/hostcert.pem',
                         'SITERM_KEY': '/etc/grid-security/hostkey.pem',
                         'SITERM_VERIFY': False}.items():
            if key not in self.config.keys():
                self.config[key] = val

    def _validateConfig(self):
        """Validate Config"""
        for param in ['SITERM_REPO', 'SITERM_CERT', 'SITERM_KEY', 'SITERM_VERIFY']:
            if param not in self.config.keys():
                raise Exception(f"Config parameter {param} is not set")

    def _getFullUrl(self, sitename, url):
        """Get Full URL"""
        if sitename not in self.fes.keys():
            raise Exception(f"Sitename {sitename} not found in SiteRM configs")
        urlmain = self.fes[sitename].rstrip('/')
        return f"{urlmain}/{url.lstrip('/')}"

    def makeRequest(self, sitename, url, **kwargs):
        """Make HTTP Request"""
        url = self._getFullUrl(sitename, url)
        if kwargs.get('verb', None) not in ['GET', 'POST', 'PUT']:
            raise Exception(f"Wrong action call {kwargs}")
        # GET
        if kwargs.get('verb') == 'GET':
            out = requests.get(url, cert=self.cert, params=kwargs.get('urlparams', None),
                               verify=self.config['SITERM_VERIFY'], timeout=getHTTPTimeout())
        # POST
        elif kwargs.get('verb') == 'POST':
            out = requests.post(url, cert=self.cert, json=kwargs.get('data', {}),
                                params=kwargs.get('urlparams', None),
                                verify=self.config['SITERM_VERIFY'], timeout=getHTTPTimeout())
        # PUT
        elif kwargs.get('verb') == 'PUT':
            out = requests.put(url, cert=self.cert, json=kwargs.get('data', {}),
                               params=kwargs.get('urlparams', None),
                               verify=self.config['SITERM_VERIFY'], timeout=getHTTPTimeout())
        outval = ""
        try:
            if out.headers.get("content-type") == "application/json":
                outval = out.json()
        except:
            outval = out.text


        return outval, out.ok, out



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""
from sense.client.requestwrapper import RequestWrapper
from sense.common import classwrapper

@classwrapper
class InstanceApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """
    def __init__(self, req_wrapper=None):
        if req_wrapper is None:
            self.client = RequestWrapper()
        else:
            self.client = req_wrapper
        self.si_uuid = None
        self._allowed_actions = [
            'provision', 'cancel', 'reprovision', 'propagate', 'commit',
            'release', 'reinstate', 'verify'
        ]

    def instance_new(self, **kwargs):  # noqa: E501
        """Generate new service instance UUID  # noqa: E501

        Retrieves UUID for new instance from backend.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_new(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            data = self.instance_get_with_http_info(**kwargs)
            self.si_uuid = data
            return data  # noqa: E501
        else:
            (data) = self.instance_get_with_http_info(**kwargs)  # noqa: E501
            self.si_uuid = data
            return data

    def instance_get_with_http_info(self, **kwargs):  # noqa: E501
        """Generate new service instance UUID  # noqa: E501

        Retrieves UUID for new instance from backend.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_get" % key)
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', '/instance')

    # TODO: check action types and only allow for combined ops
    def instance_operate(self, action, **kwargs):  # noqa: E501
        """Operate on a service instance  # noqa: E501

        Request an operation for specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_operate(si_uuid, action, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :param Operation action: Service operation requested
            * `provision` - Composite service to propagate, commit and verify instance.
            * `cancel` - Composite service to cancel, propagate, commit and verify instance.
            * `reprovision` - Composite service to reinstate, propagate, commit and verify instance.
        :param bool sync: Whether to perform the operation in a synchronous/blocking mode.
        :param bool force: Whether to force the operation on a failed instance.
        :param str intent: Intent UUID string or 'last'.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        if action not in self._allowed_actions:
            raise ValueError(
                f'operate action must of one of {self._allowed_actions}')

        if kwargs.get('async_req'):
            return self.instance_si_uuid_action_put_with_http_info(action, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_action_put_with_http_info(action, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_action_put_with_http_info(self, action,
                                                   **kwargs):  # noqa: E501
        """Operate on a service instance  # noqa: E501

        Request an operation for specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_action_put_with_http_info(si_uuid, action, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :param Operation action: Service operation requested   * `cancel` - Composite service to cancel, propagate, commit and verify instance.   * `release` - Composite service to cancel and propagate in 2-phase commit process.   * `verify` - Begin service verification process. (required)
        :param bool sync: Whether to perform the operation in a synchronous/blocking mode.
        :param bool force: Whether to force the operation on a failed instance.
        :param str intent: Intent UUID string or 'last'.
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid', 'action', 'sync', 'force', 'intent']
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_action_put" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_action_put`"
            )  # noqa: E501
        # verify the required parameter 'action' is set
        if ('action' not in params or params['action'] is None):
            raise ValueError(
                "Missing the required parameter `action` when calling `instance_si_uuid_action_put`"
            )  # noqa: E501

        query_params = []
        if 'sync' in params:
            query_params.append(('sync', params['sync']))
        if 'force' in params:
            query_params.append(('force', params['force']))
        if 'intent' in params:
            query_params.append(('intent', params['intent']))

        return self.client.request('PUT',
                                   '/instance/' + kwargs['si_uuid'] + '/' + action,
                                   query_params=query_params)

    def instance_delete(self, **kwargs):  # noqa: E501
        """Delete a service instance  # noqa: E501

        Deletes the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_delete(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_delete_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_delete_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_delete_with_http_info(self,
                                               **kwargs):  # noqa: E501
        """Delete a service instance  # noqa: E501

        Deletes the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_delete_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_delete" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_delete`"
            )  # noqa: E501

        return self.client.request('DELETE', '/instance/' + kwargs['si_uuid'])


    def instance_archive(self, **kwargs):  # noqa: E501
        """Archive a service instance  # noqa: E501

        Archive the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_archive(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_archive_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_archive_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_archive_with_http_info(self, **kwargs):  # noqa: E501
        """Archive a service instance  # noqa: E501

        Archives the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_archive_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_archive" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_archive`"
            )  # noqa: E501

        return self.client.request('PUT', '/instance/' + kwargs['si_uuid'] + '/archive')


    def instance_unarchive(self, **kwargs):  # noqa: E501
        """UnArchive a service instance  # noqa: E501

        UnArchive the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_unarchive(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_unarchive_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_unarchive_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_unarchive_with_http_info(self, **kwargs):  # noqa: E501
        """Archive a service instance  # noqa: E501

        Archives the specified service instance.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_delete_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: service instance UUID (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_unarchive" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_unarchive`"
            )  # noqa: E501

        return self.client.request('PUT', '/instance/' + kwargs['si_uuid'] + '/unarchive')


    def instance_create(self, intent, **kwargs):  # noqa: E501
        """Create new service instance and/or add new intent  # noqa: E501

        Creates a new service instance with the given UUID and intent specification or add new intent to existing service instance  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_create(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_post_with_http_info(
                intent, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_post_with_http_info(
                intent, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_post_with_http_info(self, body,
                                             **kwargs):  # noqa: E501
        """Create new service instance and/or add new intent  # noqa: E501

        Creates a new service instance with the given UUID and intent specification or add new intent to existing service instance  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_post_with_http_info(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'si_uuid', 'sync', 'proceed']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_post" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `instance_si_uuid_post`"
            )  # noqa: E501
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_post`"
            )  # noqa: E501

        path_params = {}
        if 'si_uuid' in params:
            path_params['siUUID'] = params['si_uuid']  # noqa: E501

        query_params = []
        if 'sync' in params:
            query_params.append(('sync', params['sync']))
        # sync is true by default
        if 'proceed' in params:
            query_params.append(('proceed', params['proceed']))

        # Authentication setting
        auth_settings = ['oAuth2Keycloak']  # noqa: E501
        return self.client.request('POST',
                                   '/instance/' + kwargs['si_uuid'],
                                   body_params=body,
                                   query_params=query_params)

    def instance_modify(self, intent, **kwargs):  # noqa: E501
        """Create new service instance and/or add new intent  # noqa: E501

        Creates a new service instance with the given UUID and intent specification or add new intent to existing service instance  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_modify(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_modify_post_with_http_info(
                intent, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_modify_post_with_http_info(
                intent, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_modify_post_with_http_info(self, body,
                                                    **kwargs):  # noqa: E501
        """Create new service instance and/or add new intent  # noqa: E501

        Creates a new service instance with the given UUID and intent specification or add new intent to existing service instance  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_post_with_http_info(body, si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ServiceIntent body: Service instance creation request object. (required)
        :param str si_uuid: Service instance UUID. (required)
        :return: ServiceIntentResponse
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'si_uuid', 'sync']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method instance_si_uuid_modify_post_with_http_info" %
                    key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `instance_modify`"
            )  # noqa: E501
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_modify`"
            )  # noqa: E501

        path_params = {}
        if 'si_uuid' in params:
            path_params['siUUID'] = params['si_uuid']  # noqa: E501

        query_params = []
        if 'sync' in params:
            query_params.append(('sync', params['sync']))
        # sync is true by default

        # Authentication setting
        auth_settings = ['oAuth2Keycloak']  # noqa: E501
        return self.client.request('POST',
                                   f'/instance/{kwargs["si_uuid"]}/modify',
                                   body_params=body,
                                   query_params=query_params)

    def instance_get_status(self, **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_status_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_status_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_status_get_with_http_info(self,
                                                   **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_status_get_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_si_uuid_status_get" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_si_uuid_status_get`"
            )  # noqa: E501

        return self.client.request('GET', f'/instance/{kwargs["si_uuid"]}/status')

    def instance_get_substatus(self, **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_substatus_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_substatus_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def instance_si_uuid_substatus_get_with_http_info(self,
                                                      **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_substatus_get_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_get_substatus" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_get_substatus`"
            )  # noqa: E501

        return self.client.request('GET', f'/instance/{kwargs["si_uuid"]}/substatus')

    def instance_set_superstate(self, state, **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :param str state: superstate. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.instance_si_uuid_superstate_put_with_http_info(state, **kwargs)  # noqa: E501
        else:
            (data) = self.instance_si_uuid_superstate_put_with_http_info(state, **kwargs)  # noqa: E501
            return data

    def instance_si_uuid_superstate_put_with_http_info(self, state,
                                                       **kwargs):  # noqa: E501
        """Get instance status  # noqa: E501
        Retrieves the full instance status.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.instance_si_uuid_superstate_get_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()
        :param async_req bool
        :param str si_uuid: Service instance UUID. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid', 'state']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method instance_get_superstate" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `instance_set_superstate`"
            )  # noqa: E501
        if ('state' not in params or params['state'] is None):
            raise ValueError(
                "Missing the required parameter `state` when calling `instance_set_superstate`"
            )  # noqa: E501

        return self.client.request('PUT',
                                   f'/instance/{kwargs["si_uuid"]}/superstate/{state}')

    def instance_get_intents(self, **kwargs):  # noqa: E501
        """Retrieve intents by service instance  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Intent UUID. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.intent_instance_si_uuid_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.intent_instance_si_uuid_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def intent_instance_si_uuid_get_with_http_info(self,
                                                   **kwargs):  # noqa: E501
        """Retrieve intents by service instance  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.intent_instance_si_uuid_get_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: Intent UUID. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method intent_instance_si_uuid_get" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `intent_instance_si_uuid_get`"
            )  # noqa: E501

        return self.client.request('GET', '/intent/instance/' + kwargs['si_uuid'])

    def profile_list(self, **kwargs):
        """Get skimmed profile data  # noqa: E501

        Retrieves the list of profiles the user is permitted to use without any JSON data.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :return: list[SlimProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.profile_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def profile_get_with_http_info(self, **kwargs):
        """Get skimmed profile data  # noqa: E501

        Retrieves the list of profiles the user is permitted to use without any JSON data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[SlimProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_get" % key)
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', '/profile')

    def profile_describe(self, uuid, **kwargs):  # noqa: E501
        """Get single profile  # noqa: E501

        Retrieves the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: FullProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_get_with_http_info(uuid,
                                                        **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_get_with_http_info(
                uuid, **kwargs)  # noqa: E501
            return data

    def profile_uuid_get_with_http_info(self, uuid, **kwargs):  # noqa: E501
        """Get single profile  # noqa: E501

        Retrieves the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_get_with_http_info(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: FullProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_get" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_get`"
            )  # noqa: E501

        return self.client.request('GET', '/profile/' + uuid)

    def instance_get_logging(self, **kwargs):  # noqa: E501
        """Retrieve intents by service instance  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str si_uuid: Intent UUID. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        if self.si_uuid:
            kwargs['si_uuid'] = self.si_uuid
        if not kwargs['si_uuid'] :
            raise ValueError("Missing the required parameter `si_uuid`")

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.logging_logs_si_uuid_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.logging_logs_si_uuid_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def logging_logs_si_uuid_get_with_http_info(self,
                                                **kwargs):  # noqa: E501
        """Retrieve intents by service instance  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.logging_logs_si_uuid_get_with_http_info(si_uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str si_uuid: Intent UUID. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['si_uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method logging_logs_si_uuid_get_with_http_info" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if ('si_uuid' not in params or params['si_uuid'] is None):
            raise ValueError(
                "Missing the required parameter `si_uuid` when calling `logging_logs_si_uuid_get_with_http_info`"
            )  # noqa: E501

        return self.client.request('GET', '/logging/logs/' + kwargs['si_uuid'])



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""
from sense.client.requestwrapper import RequestWrapper
from sense.common import classwrapper

@classwrapper
class ProfileApi():
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """
    def __init__(self, req_wrapper=None):
        if req_wrapper is None:
            self.client = RequestWrapper()
        else:
            self.client = req_wrapper
        if 'SI_UUID' in self.client.config:
            self.si_uuid = self.client.config['SI_UUID']
        else:
            self.si_uuid = None

    def profile_list(self, **kwargs):
        """Get skimmed profile data  # noqa: E501

        Retrieves the list of profiles the user is permitted to use without any JSON data.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :return: list[SlimProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_get_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.profile_get_with_http_info(**kwargs)  # noqa: E501
            return data

    def profile_get_with_http_info(self, **kwargs):
        """Get skimmed profile data  # noqa: E501

        Retrieves the list of profiles the user is permitted to use without any JSON data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_get_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :return: list[SlimProfile]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = []
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_get" % key)
            params[key] = val
        del params['kwargs']

        return self.client.request('GET', f'/profile')

    def profile_describe(self, desc, **kwargs):  # noqa: E501
        """Get single profile  # noqa: E501

        Retrieves the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: FullProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_search_get_with_http_info(search,
                                                        **kwargs)  # noqa: E501
        else:
            (data) = self.profile_search_get_with_http_info(
                desc, **kwargs)  # noqa: E501
            return data

    def profile_search_get_with_http_info(self, search, **kwargs):  # noqa: E501
        """Get single profile by name or uuid # noqa: E501

        Retrieves the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_search_get_with_http_info(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: FullProfile
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['search', 'force', 'fetch']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_get" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uuid' is set
        if ('search' not in params or params['search'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_get`"
            )  # noqa: E501

        query_params = []
        if 'force' in params:
            query_params.append(('force', params['force']))
        if 'fetch' in params:
            query_params.append(('fetch', params['fetch']))

        return self.client.request('GET', f'/profile/' + search,
                                   query_params=query_params)

    def profile_create(self, body, **kwargs):  # noqa: E501
        """Create a profile  # noqa: E501

        Builds and saves a new profile, using provided starting data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_create(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileManifest body: Profile creation manifest. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_post_with_http_info(body,
                                                    **kwargs)  # noqa: E501
        else:
            (data) = self.profile_post_with_http_info(body,
                                                      **kwargs)  # noqa: E501
            return data

    def profile_post_with_http_info(self, body, **kwargs):  # noqa: E501
        """Create a profile  # noqa: E501

        Builds and saves a new profile, using provided starting data.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_post_with_http_info(body, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileManifest body: Profile creation manifest. (required)
        :return: str
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_post" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `profile_post`"
            )  # noqa: E501

        body_params = None
        if 'body' in params:
            body_params = params['body']

        return self.client.request('POST',
                                   f'/profile',
                                   body_params=body_params)

    def profile_delete(self, uuid, **kwargs):  # noqa: E501
        """Delete profile  # noqa: E501

        Deletes the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_delete(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_delete_with_http_info(
                uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_delete_with_http_info(
                uuid, **kwargs)  # noqa: E501
            return data

    def profile_uuid_delete_with_http_info(self, uuid, **kwargs):  # noqa: E501
        """Delete profile  # noqa: E501

        Deletes the specified profile.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_delete_with_http_info(uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_delete" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_delete`"
            )  # noqa: E501

        return self.client.request('DELETE', f'/profile/{uuid}')

    def profile_add_licenses(self, body, uuid, **kwargs):  # noqa: E501
        """Add new license  # noqa: E501

        Assigns a new license to a user, giving them access to execute the specified profile (and potentially edit as well).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_add_licenses(body, uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileLicense body: License object. (required)
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_licenses_post_with_http_info(
                body, uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_licenses_post_with_http_info(
                body, uuid, **kwargs)  # noqa: E501
            return data

    def profile_uuid_licenses_post_with_http_info(self, body, uuid,
                                                  **kwargs):  # noqa: E501
        """Add new license  # noqa: E501

        Assigns a new license to a user, giving them access to execute the specified profile (and potentially edit as well).  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_licenses_post_with_http_info(body, uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileLicense body: License object. (required)
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_licenses_post" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `profile_uuid_licenses_post`"
            )  # noqa: E501
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_licenses_post`"
            )  # noqa: E501

        path_params = {}
        if 'uuid' in params:
            path_params['uuid'] = params['uuid']  # noqa: E501

        body_params = None
        if 'body' in params:
            body_params = params['body']

        return self.client.request('POST',
                                   f'/profile/{uuid}/licenses',
                                   body_params=body_params)

    def profile_update_licenses(self, body, uuid, **kwargs):  # noqa: E501
        """Edit existing license  # noqa: E501

        Edits an existing license to a user. Setting the remaining field to 0 will delete the license.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_update_licenses(body, uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileLicense body: License object. (required)
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_licenses_put_with_http_info(
                body, uuid, **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_licenses_put_with_http_info(
                body, uuid, **kwargs)  # noqa: E501
            return data

    def profile_uuid_licenses_put_with_http_info(self, body, uuid,
                                                 **kwargs):  # noqa: E501
        """Edit existing license  # noqa: E501

        Edits an existing license to a user. Setting the remaining field to 0 will delete the license.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_licenses_put_with_http_info(body, uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileLicense body: License object. (required)
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_licenses_put" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `profile_uuid_licenses_put`"
            )  # noqa: E501
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_licenses_put`"
            )  # noqa: E501

        path_params = {}
        if 'uuid' in params:
            path_params['uuid'] = params['uuid']  # noqa: E501

        body_params = None
        if 'body' in params:
            body_params = params['body']

        return self.client.request('PUT',
                                   f'/profile/{uuid}/licenses',
                                   body_params=body_params)

    def profile_update(self, body, uuid, **kwargs):  # noqa: E501
        """Edit a profile  # noqa: E501

        Submits an updated version of a profile for saving.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_put(body, uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileManifest body: Profile creation manifest. (required)
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_put_with_http_info(body, uuid,
                                                        **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_put_with_http_info(
                body, uuid, **kwargs)  # noqa: E501
            return data

    def profile_uuid_put_with_http_info(self, body, uuid,
                                        **kwargs):  # noqa: E501
        """Edit a profile  # noqa: E501

        Submits an updated version of a profile for saving.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_put_with_http_info(body, uuid, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param ProfileManifest body: Profile creation manifest. (required)
        :param str uuid: Profile UUID. (required)
        :return: None
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['body', 'uuid']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_put" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'body' is set
        if ('body' not in params or params['body'] is None):
            raise ValueError(
                "Missing the required parameter `body` when calling `profile_uuid_put`"
            )  # noqa: E501
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_put`"
            )  # noqa: E501

        path_params = {}
        if 'uuid' in params:
            path_params['uuid'] = params['uuid']  # noqa: E501

        body_params = None
        if 'body' in params:
            body_params = params['body']

        return self.client.request('PUT',
                                   f'/profile/{uuid}',
                                   body_params=body_params)

    def profile_get_uses(self, uuid, username, **kwargs):  # noqa: E501
        """Get license usage  # noqa: E501

        Retrieves the remaining number of tickets or slots for allocations.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_uses_username_get(uuid, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :param str username: Username of licensed user. (required)
        :return: float
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.profile_uuid_uses_username_get_with_http_info(
                uuid, username, **kwargs)  # noqa: E501
        else:
            (data) = self.profile_uuid_uses_username_get_with_http_info(
                uuid, username, **kwargs)  # noqa: E501
            return data

    def profile_uuid_uses_username_get_with_http_info(self, uuid, username,
                                                      **kwargs):  # noqa: E501
        """Get license usage  # noqa: E501

        Retrieves the remaining number of tickets or slots for allocations.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.profile_uuid_uses_username_get_with_http_info(uuid, username, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str uuid: Profile UUID. (required)
        :param str username: Username of licensed user. (required)
        :return: float
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['uuid', 'username']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method profile_uuid_uses_username_get" %
                                key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'uuid' is set
        if ('uuid' not in params or params['uuid'] is None):
            raise ValueError(
                "Missing the required parameter `uuid` when calling `profile_uuid_uses_username_get`"
            )  # noqa: E501
        # verify the required parameter 'username' is set
        if ('username' not in params or params['username'] is None):
            raise ValueError(
                "Missing the required parameter `username` when calling `profile_uuid_uses_username_get`"
            )  # noqa: E501

        path_params = {}
        if 'uuid' in params:
            path_params['uuid'] = params['uuid']  # noqa: E501
        if 'username' in params:
            path_params['username'] = params['username']  # noqa: E501

        return self.client.request('GET', f'/profile/{uuid}/uses/{username}')



#!/usr/bin/env python3

import argparse
import os
import json

from sense.client.address_api import AddressApi
from sense.client.metadata_api import MetadataApi
from sense.client.task_api import TaskApi
from sense.client.workflow_combined_api import WorkflowCombinedApi
from sense.client.profile_api import ProfileApi
from sense.client.discover_api import DiscoverApi

if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    operations = parser.add_mutually_exclusive_group()
    operations.add_argument("-cr", "--create", action="store_true",
                            help="create service instance (requires one of optional -f, optional -u)")
    operations.add_argument("-ca", "--cancel", action="store_true",
                            help="cancel (and not delete) an existing service instance (requires -u)")
    operations.add_argument("-co", "--compute", action="store_true",
                            help="compute (compile only) a service intent (requires one of optional -f, optional -u)")
    operations.add_argument("-mo", "--modify", action="store_true",
                            help="modify and an already provisioned service instance (requires -f -u)")
    operations.add_argument("-pr", "--provision", action="store_true",
                            help="provision an already computed service instance (requires -u)")
    operations.add_argument("-r", "--reprovision", action="store_true",
                            help="reprovision an existing service instance (requires -u)")
    operations.add_argument("-d", "--delete", action="store_true",
                            help="cancel and delete service instance (requires -u)")
    operations.add_argument("-D", "--delete-only", action="store_true",
                            help="delete service instance (requires -u)")
    operations.add_argument("-s", "--status", action="store_true",
                            help="get service instance status (requires -u)")
    operations.add_argument("-p", "--profile", action="store_true",
                            help="describe a service profile (requires -u)")
    operations.add_argument("-m", "--manifest", action="store_true",
                            help="create manfiest with template (requires -f -u)")
    operations.add_argument("-M", "--metadata-get", action="store_true",
                            help="Retrieve metadata record (requires --domain --name)")
    operations.add_argument("--metadata-post", action="store_true",
                            help="Add/replace metadata record (requires --domain --name --file)")
    operations.add_argument("--metadata-update", action="store_true",
                            help="Update specific metadata record fields (requires --domain --name --file)")
    operations.add_argument("--metapolicy-get", action="store_true",
                            help="Retrieve the policies of a metadata record (requires --domain --name)")
    operations.add_argument("--metapolicy-update", action="store_true",
                            help="Updates a policy of a metadata record (requires --domain --name --file)")
    operations.add_argument("--metapolicy-delete", action="store_true",
                            help="Removes a policy of a metadata record (requires --domain --name --policy)")
    operations.add_argument("-T", "--task-query", action="store_true",
                            help="Retrieve all assigned tasks (requires --assigned)")
    operations.add_argument("--task-agent-status", action="store_true",
                            help="Retrieve all assigned tasks by status (requires --assigned and --state)")
    operations.add_argument("--task-get", action="store_true",
                            help="Retrieve a specific task (requires --uuid)")
    operations.add_argument("--task-update", action="store_true",
                            help="Update a task status (requires --uuid --status). Can add an optional status JSON using --file.")
    operations.add_argument("--task-delete", action="store_true",
                            help="Remove a task (requires --uuid)")
    parser.add_argument("-f", "--file", action="append",
                        help="service intent request file")
    parser.add_argument("-u", "--uuid", action="append",
                        help="service profile uuid or instance uuid")
    parser.add_argument("-n", "--name", action="append",
                        help="service instance alias or metadata record name")
    parser.add_argument("--domain", action="append",
                        help="metadata record domain")
    parser.add_argument("--policy", action="append",
                        help="metadata policy name")
    parser.add_argument("--discover", action="append",
                        help="discover information via model query")
    parser.add_argument("--assigned", action="append",
                        help="assigned principal")
    parser.add_argument("--state", action="append",
                        help="optional status parameter")
    parser.add_argument("--address", action="append",
                        help="address (ipv4, ipv6, mac, id) allocate, free and affiliate")
    parser.add_argument("--intent", action="append",
                        help="intent UUID parameter")
    parser.add_argument("-opt", "--options", action="append",
                        help="Add additional options")
    parser.add_argument("-v", "--verbose", action="store_true",
                        help="verbose mode providing extra output")

    args = parser.parse_args()

    if args.create:
        if args.file:
            workflowApi = WorkflowCombinedApi()
            if not os.path.isfile(args.file[0]):
                workflowApi.instance_delete()
                raise Exception('request file not found: %s' % args.file[0])
            intent_file = open(args.file[0])
            intent = json.load(intent_file)
            if args.name:
                intent['alias'] = args.name[0]
            intent_file.close()
            if args.uuid:
                workflowApi.si_uuid = args.uuid[0]
                response = workflowApi.instance_create(json.dumps(intent))
            else:
                workflowApi.instance_new()
                try:
                    response = workflowApi.instance_create(json.dumps(intent))
                except ValueError:
                    workflowApi.instance_delete()
                    raise
            print(response)
            workflowApi.instance_operate('provision', sync='true')
            status = workflowApi.instance_get_status()
            print(f'provision status={status}')
        elif args.uuid:
            # create by straight profile
            intent = {'service_profile_uuid': args.uuid[0]}
            if args.name:
                intent['alias'] = args.name[0]
            workflowApi = WorkflowCombinedApi()
            workflowApi.instance_new()
            try:
                response = workflowApi.instance_create(json.dumps(intent))
                print(f"creating service instance: {response}")
            except ValueError:
                workflowApi.instance_delete()
                raise
            workflowApi.instance_operate('provision', sync='true')
            status = workflowApi.instance_get_status()
            print(f'provision status={status}')
    elif args.compute:
        if args.file:
            workflowApi = WorkflowCombinedApi()
            if not os.path.isfile(args.file[0]):
                raise Exception('request file not found: %s' % args.file[0])
            intent_file = open(args.file[0])
            intent = json.load(intent_file)
            if args.name:
                intent['alias'] = args.name[0]
            intent_file.close()
            if args.uuid:
                workflowApi.si_uuid = args.uuid[0]
                response = workflowApi.instance_create(json.dumps(intent))
            else:
                workflowApi.instance_new()
                try:
                    response = workflowApi.instance_create(json.dumps(intent))
                except ValueError:
                    workflowApi.instance_delete()
                    raise
            if not args.verbose and '"model":' in response:
                res_dict = json.loads(response)
                if 'model' in res_dict:
                    res_dict.pop('model')
                if 'queries' in res_dict:
                    res_dict.pop('queries')
                response = json.dumps(res_dict)
            print(f"computed service instance: {response}")
        elif args.uuid:
            # create by straight profile
            intent = {'service_profile_uuid': args.uuid[0]}
            if args.name:
                intent['alias'] = args.name[0]
            workflowApi = WorkflowCombinedApi()
            workflowApi.instance_new()
            try:
                response = workflowApi.instance_create(json.dumps(intent))
            except ValueError:
                workflowApi.instance_delete()
                raise
            if not args.verbose and '"model":' in response:
                res_dict = json.loads(response)
                if 'model' in res_dict:
                    res_dict.pop('model')
                if 'queries' in res_dict:
                    res_dict.pop('queries')
                response = json.dumps(res_dict)
            print(f"computed service instance: {response}")
    elif args.modify:
        if not args.file:
            raise ValueError("Missing the request file `-f mod_intent_json_file`")
        workflowApi = WorkflowCombinedApi()
        if not os.path.isfile(args.file[0]):
            raise Exception('request file not found: %s' % args.file[0])
        intent_file = open(args.file[0])
        intent = json.load(intent_file)
        intent_file.close()
        if not args.uuid:
            raise ValueError("Missing the instance uuid `-u uuid`")
        workflowApi.si_uuid = args.uuid[0]
        auto_proceed = 'true'
        if args.options:
            modifyOpts = args.options[0].split(",")
            if 'compute-only' in modifyOpts:
                auto_proceed = 'false'
        try:
            response = workflowApi.instance_modify(json.dumps(intent), sync='true', proceed=auto_proceed)
        except ValueError:
            raise
        print(f"modified service instance: {response}")
    elif args.cancel:
        if args.uuid:
            workflowApi = WorkflowCombinedApi()
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            if 'error' in status:
                raise ValueError(status)
            if 'CREATE' not in status and 'REINSTATE' not in status and 'MODIFY' not in status:
                raise ValueError(f"cannot cancel an instance in '{status}' status...")
            elif 'READY' not in status:
                workflowApi.instance_operate('cancel', si_uuid=args.uuid[0], sync='true', force='true')
            else:
                workflowApi.instance_operate('cancel', si_uuid=args.uuid[0], sync='true')
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            print(f'cancel status={status}')
            if 'CANCEL - READY' in status:
                print(f'cancel complete, use reprovision to instantiate again')
            else:
                print(f'cancel operation disrupted - instance not deleted - contact admin')
        else:
            raise ValueError("Missing the required parameter `uuid` ")
    elif args.provision:
        if args.uuid:
            workflowApi = WorkflowCombinedApi()
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            if 'error' in status:
                raise ValueError(status)
            if 'CREATE' not in status and 'REINSTATE' not in status and 'MODIFY' not in status:
                raise ValueError(f"cannot provision an instance in '{status}' status...")
            elif 'COMPILED' not in status:
                raise ValueError(f"cannot provision an instance in '{status}' status...")
            else:
                if args.intent:
                    workflowApi.instance_operate('provision', si_uuid=args.uuid[0], sync='true', intent=args.intent[0])
                else:
                    workflowApi.instance_operate('provision', si_uuid=args.uuid[0], sync='true')
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            print(f'provision status={status}')
    elif args.reprovision:
        if args.uuid:
            workflowApi = WorkflowCombinedApi()
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            if 'error' in status:
                raise ValueError(status)
            if 'CANCEL' not in status:
                raise ValueError(f"cannot reprovision an instance in '{status}' status...")
            elif 'READY' not in status:
                if args.intent:
                    workflowApi.instance_operate('reprovision', si_uuid=args.uuid[0], sync='true', force='true',
                                                 intent=args.intent[0])
                else:
                    workflowApi.instance_operate('reprovision', si_uuid=args.uuid[0], sync='true', force='true')
            else:
                if args.intent:
                    workflowApi.instance_operate('reprovision', si_uuid=args.uuid[0], sync='true',
                                                 intent=args.intent[0])
                else:
                    workflowApi.instance_operate('reprovision', si_uuid=args.uuid[0], sync='true')
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            print(f'reprovision status={status}')
    elif args.delete:
        if args.uuid:
            workflowApi = WorkflowCombinedApi()
            workflowApi.instance_operate('cancel', si_uuid=args.uuid[0], sync='true')
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            print(f'cancel status={status}')
            if 'CANCEL - READY' in status:
                workflowApi.instance_delete(si_uuid=args.uuid[0])
            else:
                print(f'cancel operation disrupted - instance not deleted - contact admin')
        else:
            raise ValueError("Missing the required parameter `uuid` ")
    elif args.delete_only:
        if args.uuid:
            workflowApi = WorkflowCombinedApi()
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0])
            if 'error' in status:
                raise ValueError(status)
            if 'CANCEL - READY' not in status:
                key = input(f"Delete an instance in '{status}' status (Y/n)?")
                if key != 'Y':
                    raise ValueError("Deletion aborted...")
            workflowApi.instance_delete(si_uuid=args.uuid[0])
        else:
            raise ValueError("Missing the required parameter `uuid` ")
    elif args.profile:
        if args.uuid:
            profileApi = ProfileApi()
            profile = profileApi.profile_describe(args.uuid[0])
            print(json.dumps(json.loads(profile), indent=2))
        elif args.name:
            profileApi = ProfileApi()
            profile_id = profileApi.profile_describe(args.name[0], force='true', fetch='false')
            print(profile_id)
        else:
            raise ValueError("Missing the required parameter `uuid` ")
    elif args.status:
        if args.uuid:
            workflowApi = WorkflowCombinedApi()
            if args.options:
                specStatus = args.options[0]
                if specStatus.lower() in ['phase', 'superstate', 'substatus', 'substate', 'configuration',
                                          'configstate']:
                    specStatus = specStatus.lower()
                else:
                    specStatus = None
            else:
                specStatus = None
            status = workflowApi.instance_get_status(si_uuid=args.uuid[0], status=specStatus, verbose=args.verbose)
            print(status)
        else:
            raise ValueError("Missing the required parameter `uuid` ")
    elif args.discover:
        discoverApi = DiscoverApi()
        discover_opts = args.discover[0].split("=")
        if discover_opts[0] == 'domain_list':
            if len(discover_opts) != 1:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_domains_get()
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'domain_info':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_domain_id_get(discover_opts[1])
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'domain_peers':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_domain_id_peers_get(discover_opts[1])
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'domain_ipv6pool':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_domain_id_ipv6pool_get(discover_opts[1])
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'service_instances':
            if len(discover_opts) != 1:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_service_instances_get()
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'lookup_name':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_lookup_name_get(discover_opts[1], search='name')
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'lookup_tag':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_lookup_name_get(discover_opts[1], search='tag')
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'lookup_address':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_lookup_name_get(discover_opts[1], search='NetworkAddress')
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'lookup_metadata':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_lookup_name_get(discover_opts[1], search='metadata')
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(json.dumps(json.loads(response), indent=2))
        elif discover_opts[0] == 'lookup_rooturi':
            if len(discover_opts) != 2:
                raise ValueError(f"Invalid discover query option `{args.discover}`")
            response = discoverApi.discover_lookup_rooturi_get(discover_opts[1])
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Discover query failed with option `{args.discover}`")
            print(str(response))
        else:
            raise ValueError(f"Invalid discover query option `{args.discover}`")
    elif args.manifest:
        if args.uuid and args.file:
            workflowApi = WorkflowCombinedApi()
            if not os.path.isfile(args.file[0]):
                raise Exception('template file not found: %s' % args.file[0])
            template_file = open(args.file[0])
            teamplate = json.load(template_file)
            template_file.close()
            workflowApi.si_uuid = args.uuid[0]
            response = workflowApi.manifest_create(json.dumps(teamplate))
            print(str(response))
        elif args.file:
            workflowApi = WorkflowCombinedApi()
            if not os.path.isfile(args.file[0]):
                raise Exception('template file not found: %s' % args.file[0])
            template_file = open(args.file[0])
            teamplate = json.load(template_file)
            template_file.close()
            workflowApi.si_uuid = None
            response = workflowApi.manifest_create(json.dumps(teamplate))
            print(str(response))
        else:
            raise ValueError(f"Invalid manifest options: require both -f josn_template and -u uuid")
    elif args.metadata_get:
        if args.domain and args.name:
            metadataAPI = MetadataApi()
            record = metadataAPI.get_metadata(domain=args.domain[0], name=args.name[0])
            print(record)
        else:
            raise ValueError(f"Invalid metadata-get options: requires -d domain and -n name")
    elif args.metadata_post:
        if args.domain and args.name and args.file:
            metadataAPI = MetadataApi()
            if not os.path.isfile(args.file[0]):
                raise Exception('data file not found: %s' % args.file[0])
            template_file = open(args.file[0])
            data = json.load(template_file)
            record = metadataAPI.post_metadata(json.dumps(data), domain=args.domain[0], name=args.name[0]),
            print(record)
        else:
            raise ValueError(f"Invalid metadata-post options: requires -d domain and -n name and -f data-file")
    elif args.metadata_update:
        if args.domain and args.name and args.file:
            metadataAPI = MetadataApi()
            if not os.path.isfile(args.file[0]):
                raise Exception('data file not found: %s' % args.file[0])
            template_file = open(args.file[0])
            data = json.load(template_file)
            record = metadataAPI.update_metadata(json.dumps(data), domain=args.domain[0], name=args.name[0]),
            print(record)
        else:
            raise ValueError(f"Invalid metadata-update options: requires -d domain and -n name and -f data-file")
    elif args.metapolicy_get:
        if args.domain and args.name:
            metadataAPI = MetadataApi()
            record = metadataAPI.get_metadata_policies(domain=args.domain[0], name=args.name[0])
            print(record)
        else:
            raise ValueError(f"Invalid metapolicy-get options: requires -d domain and -n name")
    elif args.metapolicy_update:
        if args.domain and args.name and args.file:
            metadataAPI = MetadataApi()
            if not os.path.isfile(args.file[0]):
                raise Exception('data file not found: %s' % args.file[0])
            template_file = open(args.file[0])
            data = json.load(template_file)
            record = metadataAPI.update_metadata_policy(json.dumps(data), domain=args.domain[0], name=args.name[0]),
            print(record)
        else:
            raise ValueError(f"Invalid metapolicy-update options: requires -d domain and -n name and -f data-file")
    elif args.metapolicy_delete:
        if args.domain and args.name and args.policy:
            metadataAPI = MetadataApi()
            record = metadataAPI.delete_metadata_policy(domain=args.domain[0], name=args.name[0], policy=args.policy[0])
            print(record)
        else:
            raise ValueError(f"Invalid metapolicy-delete options: requires --domain and --name and --policy")
    elif args.task_query:
        if args.assigned:
            taskAPI = TaskApi()
            record = taskAPI.get_tasks(assigned=args.assigned[0])
            print(record)
        else:
            raise ValueError(f"Invalid task_query options: requires --assigned")
    elif args.task_agent_status:
        if args.assigned and args.state:
            taskAPI = TaskApi()
            record = taskAPI.get_tasks_agent_status(assigned=args.assigned[0], status=args.state[0])
            print(record)
        else:
            raise ValueError(f"Invalid task_query options: requires --assigned and --state")
    elif args.task_get:
        if args.uuid:
            taskAPI = TaskApi()
            record = taskAPI.get_task(uuid=args.uuid[0])
            print(record)
        else:
            raise ValueError(f"Invalid task_get options: requires --uuid")
    elif args.task_update:
        if args.uuid and args.state:
            taskAPI = TaskApi()
            if args.file:
                if not os.path.isfile(args.file[0]):
                    raise Exception('data file not found: %s' % args.file[0])
                template_file = open(args.file[0])
                data = json.load(template_file)
            else:
                data = None
            record = taskAPI.update_task(json.dumps(data), uuid=args.uuid[0], state=args.state[0])
            print(record)
        else:
            raise ValueError(f"Invalid task_update options: requires --uuid and --state")
    elif args.task_delete:
        if args.uuid:
            taskAPI = TaskApi()
            record = taskAPI.delete_task(uuid=args.uuid[0])
            print(record)
        else:
            raise ValueError(f"Invalid metapolicy-delete options: requires --domain and --name and --policy")
    elif args.address:
        addressApi = AddressApi()
        address_opts = args.address[0].split(",")
        params = {}
        if address_opts[0] == 'allocate':
            for i in range(1, len(address_opts)):
                kv = address_opts[i].split('=')
                if len(kv) != 2:
                    raise ValueError(f"Invalid address allocate option field:`{address_opts[i]}`")
                params[kv[0]] = kv[1]
            if 'pool' not in params:
                raise ValueError(f"Missing a 'pool' paramter in the allocate option")
            if 'type' not in params:
                raise ValueError(f"Missing a 'type' paramter in the allocate option")
            if 'name' not in params:
                raise ValueError(f"Missing a 'name' paramter in the allocate option")
            pool = params['pool']
            del params['pool']
            atype = params['type']
            del params['type']
            name = params['name']
            del params['name']
            response = addressApi.allocate_address(pool, atype, name, **params)
            if len(response) == 0 or "ERROR" in response:
                raise ValueError(f"Address allocate failed with option `{args.address}`")
            print(response)
        elif address_opts[0] == 'free':
            for i in range(1, len(address_opts)):
                kv = address_opts[i].split('=')
                if len(kv) != 2:
                    raise ValueError(f"Invalid address free option field:`{address_opts[i]}`")
                params[kv[0]] = kv[1]
            if 'pool' not in params:
                raise ValueError(f"Missing a 'pool' paramter in the free option")
            pool = params['pool']
            del params['pool']
            response = addressApi.free_address(pool, **params)
            if "ERROR" in response:
                raise ValueError(f"Address free failed with option `{args.address}`")
            print(response)
        elif address_opts[0] == 'affiliate':
            for i in range(1, len(address_opts)):
                kv = address_opts[i].split('=')
                if len(kv) != 2:
                    raise ValueError(f"Invalid address affiliate option field:`{address_opts[i]}`")
                params[kv[0]] = kv[1]
            if 'pool' not in params:
                raise ValueError(f"Missing a 'pool' paramter in the affiliate option")
            if 'uri' not in params:
                raise ValueError(f"Missing a 'uri' paramter in the affiliate option")
            pool = params['pool']
            del params['pool']
            uri = params['uri']
            del params['uri']
            response = addressApi.affiliate_address(pool, uri, **params)
            if "ERROR" in response:
                raise ValueError(f"Address affiliate failed with option `{args.address}`")
            print(response)
        else:
            raise ValueError(f"Invalid address allocate/free/affiliate options")



import os
import unittest
import pytest
import re
import json
import time
import sys

# Append root for proper imports
sys.path.append('')
#

#
from sense.client.profile_api import ProfileApi
from sense.common import loadJSON
#


class TestProfileApi(unittest.TestCase):
    @classmethod
    def preOpCreate(cls):
        paths = [
            "requests/profile-1.json", "requests/profile-2.json"
        ]
        for path in paths:
            profile = loadJSON(path)
            res = cls.client.profile_create(json.dumps(profile))
            cls.uuids.append(res)

    @classmethod
    def setUpClass(cls):
        cls.uuids = []
        cls.client = ProfileApi()
        cls.preOpCreate()
        print(f"\nProfile Test :: using dummy instance: {cls.uuids[0]}")

    @classmethod
    def tearDownClass(cls):
        print(f'Profile Test :: cleaning up {len(cls.uuids)} dummy instances.')
        for uuid in cls.uuids:
            cls.client.profile_delete(uuid)

    def setUp(self) -> None:
        self.client = ProfileApi()

    def tearDown(self) -> None:
        pass

    def test_create_profile(self):
        # - TESTING: POST /
        profile = loadJSON("requests/profile-1.json")
        res = self.client.profile_create(json.dumps(profile))
        # print(f'ADD: {res}')
        assert re.match(
            '[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}',
            res)
        TestProfileApi.uuids.append(res)

    def test_get_profiles(self):
        # PRE-OP: load profile from class cache.
        uuid = TestProfileApi.uuids[0]
        #

        #
        # - TESTING: GET /
        res = json.loads(self.client.profile_list())
        for prof in res:
            if prof.get('uuid') == uuid:
                found = True
                break
        else:
            found = False
        assert found

        #
        # - TESTING: GET /{uuid}
        res = self.client.profile_describe(uuid)
        prof = json.loads(res)
        assert prof.get('uuid') == uuid

    def test_delete_profile(self):
        # PRE-OP: create profile for deletion test.
        self.test_create_profile()
        uuid = TestProfileApi.uuids.pop()
        #

        #
        # - TESTING: DELETE /{uuid}
        self.client.profile_delete(uuid)
        res = self.client.profile_describe(uuid)
        assert not res

    def test_update_profile(self):
        # PRE-OP: create profile for modification test.
        self.test_create_profile()
        uuid = TestProfileApi.uuids[-1]
        #

        #
        # - TESTING: PUT /{uuid}
        input = loadJSON("requests/profile-2.json")
        self.client.profile_update(json.dumps(input), uuid)
        prof = json.loads(self.client.profile_describe(uuid))
        assert prof.get('name') == input.get('name')

    def test_profile_uses(self):
        # PRE-OP: load profile from class cache.
        uuid = TestProfileApi.uuids[0]
        #
        #
        # - TESTING: GET /{uuid}/uses/{username}
        res = self.client.profile_get_uses(uuid, "admin")
        assert res == '0'

    def test_profile_license(self):
        # PRE-OP: create profile for deletion test.
        self.test_create_profile()
        uuid = TestProfileApi.uuids[-1]
        #

        #
        # - TESTING: POST /{uuid}/licenses
        license = loadJSON("requests/profile-license-1.json")
        self.client.profile_add_licenses(json.dumps(license), uuid)
        prof = json.loads(self.client.profile_describe(uuid))
        assert len(prof.get('licenses')) == 1

        #
        # - TESTING: PUT /{uuid}/licenses
        license.update({"remaining": 99})
        self.client.profile_update_licenses(json.dumps(license), uuid)
        prof = json.loads(self.client.profile_describe(uuid))
        assert prof.get('licenses')[0].get('remaining') == 99

        license.update({"remaining": 0})
        self.client.profile_update_licenses(json.dumps(license), uuid)
        prof = json.loads(self.client.profile_describe(uuid))
        assert len(prof.get('licenses')) == 0


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    SENSE-O Northbound Intent API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ConnectionBandwidth(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'qos_class': 'str',
        'capacity': 'str'
    }

    attribute_map = {
        'qos_class': 'qos_class',
        'capacity': 'capacity'
    }

    def __init__(self, qos_class='bestEffort', capacity='1024'):  # noqa: E501
        """ConnectionBandwidth - a model defined in Swagger"""  # noqa: E501
        self._qos_class = None
        self._capacity = None
        self.discriminator = None
        if qos_class is not None:
            self.qos_class = qos_class
        if capacity is not None:
            self.capacity = capacity

    @property
    def qos_class(self):
        """Gets the qos_class of this ConnectionBandwidth.  # noqa: E501

        Quality of Service (QoS) network priority.  # noqa: E501

        :return: The qos_class of this ConnectionBandwidth.  # noqa: E501
        :rtype: str
        """
        return self._qos_class

    @qos_class.setter
    def qos_class(self, qos_class):
        """Sets the qos_class of this ConnectionBandwidth.

        Quality of Service (QoS) network priority.  # noqa: E501

        :param qos_class: The qos_class of this ConnectionBandwidth.  # noqa: E501
        :type: str
        """
        allowed_values = ["guaranteedCapped", "softCapped", "anyAvailable", "bestEffort"]  # noqa: E501
        if qos_class not in allowed_values:
            raise ValueError(
                "Invalid value for `qos_class` ({0}), must be one of {1}"  # noqa: E501
                .format(qos_class, allowed_values)
            )

        self._qos_class = qos_class

    @property
    def capacity(self):
        """Gets the capacity of this ConnectionBandwidth.  # noqa: E501

        Bandwidth limit in Mbps.  # noqa: E501

        :return: The capacity of this ConnectionBandwidth.  # noqa: E501
        :rtype: str
        """
        return self._capacity

    @capacity.setter
    def capacity(self, capacity):
        """Sets the capacity of this ConnectionBandwidth.

        Bandwidth limit in Mbps.  # noqa: E501

        :param capacity: The capacity of this ConnectionBandwidth.  # noqa: E501
        :type: str
        """

        self._capacity = capacity

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ConnectionBandwidth, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ConnectionBandwidth):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import json
from types import SimpleNamespace

from sense.client.discover_api import DiscoverApi
from sense.client.profile_api import ProfileApi
from sense.client.workflow_combined_api import WorkflowCombinedApi
from sense.workflow.base.utils import get_logger
from .sense_constants import *
from .sense_exceptions import SenseException

logger = get_logger()


def describe_profile(*, client, uuid: str):
    profile_api = ProfileApi(req_wrapper=client)
    profile_details = profile_api.profile_describe(uuid)
    profile_details = json.loads(json.dumps(profile_details), object_hook=lambda dct: SimpleNamespace(**dct))

    if hasattr(profile_details, "edit"):
        from requests import utils

        for ns in profile_details.edit:
            ns.path = utils.unquote(ns.path)

            if hasattr(ns, 'valid'):
                ns.valid = utils.unquote(ns.valid)

    return profile_details


def get_profile_uuid(*, client, profile):
    profile_api = ProfileApi(req_wrapper=client)

    try:
        profile = profile_api.profile_search_get_with_http_info(search=profile)
        profile_uuid = profile['uuid']
        return profile_uuid
    except Exception as e:
        raise SenseException(f"Exception searching for profile:{e}")


def create_instance(*, client, profile, alias, edit_template):
    profile_uuid = get_profile_uuid(client=client, profile=profile)
    workflow_api = WorkflowCombinedApi(req_wrapper=client)
    workflow_api.instance_new()
    intent = {SENSE_PROFILE_UID: profile_uuid, "alias": alias}
    profile_details = describe_profile(client=client, uuid=profile_uuid)

    logger.debug(f'Profile Details: {profile_details}')

    if hasattr(profile_details, "edit"):
        edit_entries = profile_details.edit
        temp_entries = [e.__dict__ for e in edit_entries]
        logger.info(f'Edit Entries: {json.dumps(temp_entries, indent=2)}')

    options = []

    for k, v in edit_template.items():
        options.append({k: str(v)})

    if options:
        query = dict([("ask", "edit"), ("options", options)])
        intent["queries"] = [query]

    logger.info(f'Intent: {json.dumps(intent, indent=2)}')
    intent = json.dumps(intent)
    logger.info(f"creating instance: {alias}")
    response = workflow_api.instance_create(intent, async_req=True)
    return response['service_uuid']


def instance_operate(*, client, si_uuid, action='provision'):
    import time
    from random import randint

    workflow_api = WorkflowCombinedApi(req_wrapper=client)
    status = workflow_api.instance_get_status(si_uuid=si_uuid)

    if "CREATE - COMMITTING" not in status or 'CANCEL - READY' == status:
        try:
            time.sleep(randint(5, 10))
            workflow_api.instance_operate(action, si_uuid=si_uuid, async_req=True)
        except Exception as e:
            logger.warning(f"exception from instance_operate {e}")


def wait_for_instance_create(*, client, si_uuid):
    import time
    from random import randint

    workflow_api = WorkflowCombinedApi(req_wrapper=client)

    for attempt in range(SENSE_RETRY):
        try:
            status = workflow_api.instance_get_status(si_uuid=si_uuid)
            logger.info(f"Waiting on CREATED COMPILED: status={status}:attempt={attempt} out of {SENSE_RETRY}")

            if status in ['CREATE - COMPILED']:
                break

            if 'FAILED' in status:
                break
        except Exception as e:
            logger.warning(f"exception from  instance_get_status {e}")
            pass

        logger.info(f"Waiting on CREATE COMPILED: going to sleep attempt={attempt}")
        time.sleep(randint(30, 35))

    return workflow_api.instance_get_status(si_uuid=si_uuid)


def wait_for_instance_operate(*, client, si_uuid):
    import time
    from random import randint

    workflow_api = WorkflowCombinedApi(req_wrapper=client)

    for attempt in range(SENSE_RETRY):
        try:
            status = workflow_api.instance_get_status(si_uuid=si_uuid)

            if status in ['CREATE - READY', 'REINSTATE - READY', 'MODIFY - READY']:
                return status

            logger.info(f"Waiting on CREATE/REINSTATE/MODIFY-READY: status={status}:attempt={attempt} out of {SENSE_RETRY}")

            if 'FAILED' in status:
                break
        except Exception as e:
            logger.warning(f"exception from  instance_get_status {e}")
            pass

        logger.info(f"Waiting on CREATE/REINSTATE-READY: going to sleep attempt={attempt}")
        time.sleep(randint(30, 35))

    return workflow_api.instance_get_status(si_uuid=si_uuid)


def delete_instance(*, client, si_uuid):
    # import time
    # import random

    workflow_api = WorkflowCombinedApi(req_wrapper=client)
    status = workflow_api.instance_get_status(si_uuid=si_uuid)

    if 'error' in status:
        raise SenseException("error deleting got " + status)

    if 'FAILED' in status:
        instance_dict = service_instance_details(client=client, si_uuid=si_uuid)
        lastState = instance_dict['lastState']

        if lastState in ['INIT', 'COMPILED']:
            return

        raise SenseException(f'cannot delete instance - contact admin. {status}')

    if "CREATE - COMPILED" in status:
        return

    if "CANCEL" not in status:
        if 'CREATE' not in status and 'REINSTATE' not in status and 'MODIFY' not in status:
            raise ValueError(f"cannot cancel an instance in '{status}' status...")

        # TODO REVISIT THIS. time.sleep(random.randint(5, 30))

        try:
            if 'READY' not in status:
                workflow_api.instance_operate('cancel', si_uuid=si_uuid, async_req=True, force=True)
            else:
                workflow_api.instance_operate('cancel', si_uuid=si_uuid, async_req=True)
        except ValueError as ve:
            status = workflow_api.instance_get_status(si_uuid=si_uuid)

            if "CANCEL" not in status:
                raise ve


def wait_for_delete_instance(*, client, si_uuid, alias):
    import time
    import random

    workflow_api = WorkflowCombinedApi(req_wrapper=client)
    status = workflow_api.instance_get_status(si_uuid=si_uuid)

    if 'error' in status:
        raise SenseException("error deleting got " + status)

    if 'FAILED' in status:
        instance_dict = service_instance_details(client=client, si_uuid=si_uuid)
        lastState = instance_dict['lastState']

        if lastState in ['INIT', 'COMPILED']:
            workflow_api.instance_delete(si_uuid=si_uuid)
            return

        raise SenseException(f'cannot delete instance - contact admin. {status}')
    elif "CREATE - COMPILED" in status:
        time.sleep(random.randint(5, 30))
        workflow_api.instance_delete(si_uuid=si_uuid)
        return

    for attempt in range(SENSE_RETRY):
        # This sleep is here to workaround issue where CANCEL-READY shows up prematurely.
        time.sleep(random.randint(30, 35))

        status = workflow_api.instance_get_status(si_uuid=si_uuid)
        logger.info(f"Waiting on CANCEL-READY for {alias}: status={status}:attempt={attempt} out of {SENSE_RETRY}")

        if 'CANCEL - READY' in status:  # This got triggered very quickly ...
            break

        if 'FAILED' in status:
            break

    if 'CANCEL - READY' in status:
        logger.info(f"Deleting instance: {si_uuid}")
        # TODO REVISIT THIS. time.sleep(random.randint(5, 30))
        ret = workflow_api.instance_delete(si_uuid=si_uuid)
        logger.info(f"Deleted instance: {si_uuid}: ret={ret}")
    else:
        raise SenseException(f'cancel operation disrupted - instance not deleted - contact admin. {status}')


def instance_get_status(*, client, si_uuid):
    workflow_api = WorkflowCombinedApi(req_wrapper=client)
    return workflow_api.instance_get_status(si_uuid=si_uuid)


def service_instance_details(*, client, si_uuid):
    discover_api = DiscoverApi(req_wrapper=client)
    response = discover_api.discover_service_instances_get()
    instances = response['instances']

    for instance in instances:
        temp = SimpleNamespace(**instance)

        if temp.referenceUUID == si_uuid:
            instance['intents'] = []

            for intent in temp.intents:
                intent['json'] = json.loads(intent['json'])
                instance['intents'].append(intent)

            return instance

    raise SenseException('no details found')


# TODO Handle error
# /discover/service/instances?search=test-gcp-vms
def find_instance_by_alias(*, client, alias):
    discover_api = DiscoverApi(req_wrapper=client)
    response = discover_api.discover_service_instances_get(search=alias)
    instances = response['instances']

    if not instances:
        return None

    instance = instances[0]
    return instance['referenceUUID']


def manifest_create(*, client, template, alias=None, si_uuid=None):
    import time
    from json.decoder import JSONDecodeError

    si_uuid = si_uuid or find_instance_by_alias(client=client, alias=alias)
    workflow_api = WorkflowCombinedApi(req_wrapper=client)
    template = json.dumps(template)

    for attempt in range(SENSE_RETRY):
        response = workflow_api.manifest_create(template, si_uuid=si_uuid)

        if isinstance(response, dict) and 'jsonTemplate' in response:
            try:
                details = json.loads(response['jsonTemplate'])
                return details
            except JSONDecodeError:
                logger.warning(f"Could not decode sense manifest from response={response}")

        logger.warning(f"got {response} while retrieving manifest. Will retry ....")
        time.sleep(10)

    raise SenseException(f"Unable to retrieve manifest using {template}")



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-O Northbound Address API

    StackV SENSE-O Northbound REST API Documentation  # noqa: E501

    OpenAPI spec version: 2.0.2

    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""
import json
from sense.client.requestwrapper import RequestWrapper
from sense.common import classwrapper

@classwrapper
class AddressApi():
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, req_wrapper=None):
        if req_wrapper is None:
            self.client = RequestWrapper()
        else:
            self.client = req_wrapper

    def get_allocations(self, pool: str, **kwargs):  # noqa: E501
        """Retrieve allocations by pool and scope  # noqa: E501

        Queries all service intents belonging to given instance UUID.  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str pool: Pool Name. (required)
        :return: list[IntentExpanded]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.allocations_get_with_http_info(pool, **kwargs)  # noqa: E501
        else:
            (data) = self.allocations_get_with_http_info(pool, **kwargs)  # noqa: E501
            return data

    def allocations_get_with_http_info(self, pool: str,
                                       **kwargs):  # noqa: E501
        """Retrieve allocations by pool and scope  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.allocations_get_with_http_info(pool, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pool: Pool Name (required)
        :return: list[Allocations]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['pool', 'scope']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method allocations_get_with_http_info" % key)
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'si_uuid' is set
        if 'pool' not in params or params['pool'] is None:
            raise ValueError(
                "Missing the required parameter `pool` when calling `allocations_get_with_http_info`"
            )  # noqa: E501
        if 'scope' not in params:
            params['scope'] = 'default'
        return self.client.request('GET', '/address/allocate/' + pool + '/' + params['scope'])

    def allocate_ipv4_address(self, pool: str, alloc_name: str, batch: str, scope='default'):
        return self.allocate_address(pool, 'IPv4', alloc_name, batch=batch, scope=scope)

    def allocate_ipv4_subnet(self, pool: str, alloc_name: str, netmask: str, scope='default'):
        return self.allocate_address(pool, 'IPv4', alloc_name, batch='subnet', netmask=netmask, scope=scope)

    def allocate_ipv4_address_subnet(self, pool: str, alloc_name: str, batch: str, netmask: str, scope='default'):
        return self.allocate_address(pool, 'IPv4', alloc_name, batch=batch, netmask=netmask, scope=scope)

    def allocate_ipv6_address(self, pool: str, alloc_name: str, batch: str, scope='default'):
        return self.allocate_address(pool, 'IPv6', alloc_name, batch=batch, scope=scope)

    def allocate_ipv6_subnet(self, pool: str, alloc_name: str, netmask: str, scope='default'):
        return self.allocate_address(pool, 'IPv6', alloc_name, batch='subnet', netmask=netmask, scope=scope)

    def allocate_ipv6_address_subnet(self, pool: str, alloc_name: str, batch: str, netmask: str, scope='default'):
        return self.allocate_address(pool, 'IPv6', alloc_name, batch=batch, netmask=netmask, scope=scope)

    def allocate_mac_address(self, pool: str, alloc_name: str, batch: str, scope='default'):
        return self.allocate_address(pool, 'MAC', alloc_name, batch=batch, scope=scope)

    def allocate_id_address(self, pool: str, alloc_name: str, batch: str, scope='default'):
        return self.allocate_address(pool, 'ID', alloc_name, batch=batch, scope=scope)

    def allocate_address(self, pool: str, addr_type: str, alloc_name: str, **kwargs):  # noqa: E501
        """allocate address by pool, scope and name  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str pool: Pool Name. (required)
        :param str alloc_name: Alloc Name. (required)
        :return: allocation result
                 If the method is called asynchronously,
                 returns the request thread.
        """

        kwargs['_return_http_data_only'] = True

        if addr_type not in ['IPv4', 'IPv6', 'MAC', 'ID']:
            raise ValueError(
                "Parameter 'type' must of one of ['ipv4', 'ipv6', 'mac', 'id']"
            )  # noqa: E501
        if 'scope' not in kwargs:
            scope = 'default'
        else:
            scope = kwargs['scope']
        if 'batch' not in kwargs:
            batch = 1
        else:
            batch = kwargs['batch']
            if batch.isnumeric():
                batch = int(batch)
            elif batch != 'subnet':
                raise ValueError(
                    "Parameter `batch` must be either numeric or 'subnet'"
                )  # noqa: E501
        allocRequest = {'type': addr_type, 'name': alloc_name}

        allocParams = {'scope': scope, 'batch': batch}
        if batch == 'subnet':
            if 'netmask' not in kwargs:
                raise ValueError(
                    "Missing the required parameter `netmask` when calling `allocate_post_with_http_info` for 'subnet' alloc"
                )  # noqa: E501
            allocParams['netmask'] = kwargs['netmask']
        elif not isinstance(batch, int) or batch < 1:
            raise ValueError(
                "Parameter `batch` must be either numeric or 'subnet'"
            )  # noqa: E501
        else:
            if 'netmask' in kwargs:
                allocParams['netmask'] = kwargs['netmask']
        allocRequest['allocate'] = allocParams

        if kwargs.get('async_req'):
            return self.allocate_post_with_http_info(pool, json.dumps(allocRequest), **kwargs)  # noqa: E501
        else:
            (data) = self.allocate_post_with_http_info(pool, json.dumps(allocRequest), **kwargs)  # noqa: E501
            return data

    def allocate_post_with_http_info(self, pool: str, body: str,
                                     **kwargs):  # noqa: E501
        """allocate address by pool, scope and name  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.allocate_post_with_http_info(pool, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pool: Pool Name (required)
        :return: list[Allocations]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['pool', 'scope', 'type', 'name', 'batch', 'netmask']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method intent_instance_si_uuid_get" % key)
            params[key] = val
        del params['kwargs']

        path_params = {}
        query_params = []

        return self.client.request('POST',
                                   '/address/allocate/' + pool,
                                   body_params=body)

    def free_address(self, pool: str, **kwargs):  # noqa: E501
        """free address by pool by scope and optionally with address or name  # noqa: E501

        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str pool: Pool Name. (required)
        :return: allocation result
                 If the method is called asynchronously,
                 returns the request thread.
        """

        kwargs['_return_http_data_only'] = True

        if 'scope' not in kwargs:
            kwargs['scope'] = 'default'

        if kwargs.get('async_req'):
            return self.free_delete_with_http_info(pool, **kwargs)  # noqa: E501
        else:
            (data) = self.free_delete_with_http_info(pool, **kwargs)  # noqa: E501
            return data

    def free_delete_with_http_info(self, pool: str,
                                   **kwargs):  # noqa: E501
        """free address by pool by scope and optionally with address or name  # noqa: E501

        Queries all allocations belonging to given pool and scope.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.free_delete_with_http_info(pool, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pool: Pool Name (required)
        :return: list[Allocations]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['pool', 'scope', 'name', 'address']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method intent_instance_si_uuid_get" % key)
            params[key] = val
        del params['kwargs']

        if 'pool' not in params or params['pool'] is None:
            raise ValueError(
                "Missing the required parameter `pool` when calling `allocations_get_with_http_info`"
            )  # noqa: E501
        if 'scope' not in params:
            params['scope'] = 'default'

        if 'address' not in params and 'name' not in params:
            return self.client.request('DELETE',
                                       '/address/allocate/' + pool + '/' + params['scope'])
        elif 'address' in params:
            address = params['address'].replace('/', '%2F')
            return self.client.request('DELETE',
                                       '/address/allocate/' + pool + '/' + address + '/' + params['scope'])
        else:
            return self.client.request('DELETE',
                                       '/address/allocate/' + pool + '/name/' + params['name'] + '/' + params['scope'])

    def affiliate_address(self, pool: str, uri: str, **kwargs):  # noqa: E501
        """affilicate address with resource uri by address or name  # noqa: E501
        This method makes a synchronous HTTP request by default.
        :param async_req bool
        :param str pool: Pool Name. (required)
        :return: allocation result
                 If the method is called asynchronously,
                 returns the request thread.
        """

        kwargs['_return_http_data_only'] = True

        if 'scope' not in kwargs:
            kwargs['scope'] = 'default'

        if kwargs.get('async_req'):
            return self.affiliate_put_with_http_info(pool, uri, **kwargs)  # noqa: E501
        else:
            (data) = self.affiliate_put_with_http_info(pool, uri, **kwargs)  # noqa: E501
            return data

    def affiliate_put_with_http_info(self, pool: str, uri: str,
                                     **kwargs):  # noqa: E501
        """affilicate address with resource uri by address or name  # noqa: E501
        Queries all allocations belonging to given pool and scope.  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.affiliate_put_with_http_info(pool, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str pool: Pool Name (required)
        :param str uri: Affiliated resource URI (required)
        :return: list[Allocations]
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['pool', 'scope', 'name', 'address', 'uri']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in params['kwargs'].items():
            if key not in all_params:
                raise TypeError("Got an unexpected keyword argument '%s'"
                                " to method intent_instance_si_uuid_get" % key)
            params[key] = val
        del params['kwargs']

        if 'pool' not in params or params['pool'] is None:
            raise ValueError(
                "Missing the required parameter `pool` when calling `allocations_get_with_http_info`"
            )  # noqa: E501
        if 'scope' not in params:
            params['scope'] = 'default'

        if 'address' not in params and 'name' not in params:
            raise TypeError("affiliate_address call needs either a 'name' or 'address' parameter")
        elif 'address' in params:
            address = params['address'].replace('/', '%2F')
            return self.client.request('PUT',
                                       '/address/affiliate/' + pool + '/' + address + '/' + params['scope'] + '/' + uri)
        else:
            return self.client.request('PUT',
                                       '/address/affiliate/' + pool + '/name/' + params['name'] + '/' + params[
                                           'scope'] + '/' + uri)



import os
import unittest
import pytest
import re
import json
import time
import sys

# Append root for proper imports
sys.path.append('')
#

#
from sense.client.discover_api import DiscoverApi
from sense.common import loadJSON
#


class TestDiscoverApi(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        res = json.loads(DiscoverApi().discover_get())
        cls.uri = res["domains"][0]["domain_uri"]
        print(f"\nDiscover Test :: using domain URI: {cls.uri}")

    def setUp(self) -> None:
        self.client = DiscoverApi()

    def tearDown(self) -> None:
        pass

    def test_get_full(self):
        # - TESTING: GET /discover
        res = json.loads(self.client.discover_get())
        assert "domains" in res

    def test_get_domains(self):
        # - TESTING: GET /discover/domains
        res = json.loads(self.client.discover_domains_get())
        assert "domains" in res

    def test_get_by_id(self):
        # - TESTING: GET /discover/{domainID}
        res = json.loads(
            self.client.discover_domain_id_get(TestDiscoverApi.uri))
        assert res["domain_uri"] == TestDiscoverApi.uri

    def test_get_by_id_peers(self):
        # - TESTING: GET /discover/{domainID}/peers
        res = json.loads(
            self.client.discover_domain_id_peers_get(TestDiscoverApi.uri))
        assert res["domain_uri"] == TestDiscoverApi.uri
        assert "peer_points" in res

    def test_get_instances(self):
        # - TESTING: GET /discover/service/instance
        res = json.loads(self.client.discover_service_instances_get())
        assert "instances" in res
        # TODO: leverage instance test suite to create dummy for query testing.

    @unittest.expectedFailure
    def test_lookup(self):
        # TODO: find better/consistent way to get correct query name. Expected to fail otherwise.
        # - TESTING: GET /discover/lookup/{name}
        res = json.loads(
            self.client.discover_lookup_name_get(TestDiscoverApi.uri))
        assert "results" in res



from typing import List, Dict

from sense.workflow.base.constants import Constants
from sense.workflow.base.exceptions import ProviderTypeNotSupported, ProviderException
from sense.workflow.provider.provider import Provider


class ProviderFactory:
    def __init__(self):
        self._providers: Dict[str, Provider] = {}

    # noinspection PyBroadException
    def init_provider(self, *, type: str, label: str, name: str, attributes) -> Provider:
        if type not in Constants.PROVIDER_CLASSES:
            raise ProviderTypeNotSupported(type)

        import importlib

        full_name = Constants.PROVIDER_CLASSES.get(type)
        idx = full_name.rindex('.')
        module_name = full_name[:idx]
        class_name = full_name[idx+1:]
        cls = getattr(importlib.import_module(module_name), class_name)
        provider = cls(type=type, label=label, name=name, config=attributes)

        try:
            provider.init()
        except Exception as e:
            raise ProviderException(f"Exception encountered while initializing {label}: {e}")

        self._providers[label] = provider
        return provider

    @property
    def providers(self) -> List[Provider]:
        return list(self._providers.values())

    def has_provider(self, *, label: str) -> bool:
        return label in self._providers

    def get_provider(self, *, label: str) -> Provider:
        return self._providers[label]


default_provider_factory = ProviderFactory()



#!/usr/bin/env python3
"""Request Wrapper for SENSE-0 API"""
import sys

import logging
from http.client import HTTPConnection
import requests
from sense.client.apiclient import ApiClient
from sense.common import isDebugSet
from sense.common import getHTTPTimeout
from sense.common import getHTTPRetries
from sense.common import classwrapper

sys.path.insert(0, '..')


@classwrapper
class RequestWrapper(ApiClient):
    """Request Wrapper for SENSE-0 API (GET, PUT, POST, DELETE)"""
    def __init__(self, config=None):
        self.logger = None
        self.__setdebug()
        super(RequestWrapper, self).__init__(config)

    def __setdebug(self):
        """Set Debug and log all http call details to console"""
        handler = logging.StreamHandler()
        formatter = logging.Formatter(
            "%(asctime)s.%(msecs)03d - %(name)s - %(levelname)s - %(message)s",
            datefmt="%a, %d %b %Y %H:%M:%S",
        )
        handler.setFormatter(formatter)

        logLevel = logging.INFO
        if isDebugSet():
            logLevel = logging.DEBUG
            HTTPConnection.debuglevel = 1

        self.logger = logging.getLogger('RequestWrapperSense')
        if not self.logger.handlers:
            self.logger.addHandler(handler)
        self.logger.setLevel(logLevel)


    def _get(self, api_path, params):
        url = self.config['REST_API'] + api_path
        out = requests.get(url,
                           headers=self.config['headers'],
                           verify=self.config['verify'],
                           params=params, timeout=getHTTPTimeout())
        if out.status_code == 401:
            self._refreshToken()
            out = requests.get(url,
                               headers=self.config['headers'],
                               verify=self.config['verify'],
                               params=params, timeout=getHTTPTimeout())
        return out

    def _put(self, api_path, data, params):
        url = self.config['REST_API'] + api_path
        out = requests.put(url,
                           headers=self.config['headers'],
                           verify=self.config['verify'],
                           data=data,
                           params=params, timeout=getHTTPTimeout())
        if out.status_code == 401:
            self._refreshToken()
            out = requests.put(url,
                               headers=self.config['headers'],
                               verify=self.config['verify'],
                               data=data,
                               params=params, timeout=getHTTPTimeout())
        return out

    def _post(self, api_path, data, params):
        url = self.config['REST_API'] + api_path
        out = requests.post(url,
                            headers=self.config['headers'],
                            verify=self.config['verify'],
                            data=data,
                            params=params, timeout=getHTTPTimeout())
        if out.status_code == 401:
            self._refreshToken()
            out = requests.post(url,
                                headers=self.config['headers'],
                                verify=self.config['verify'],
                                data=data,
                                params=params, timeout=getHTTPTimeout())
        return out

    def _delete(self, api_path, params):
        url = self.config['REST_API'] + api_path
        out = requests.delete(url,
                              headers=self.config['headers'],
                              verify=self.config['verify'],
                              params=params, timeout=getHTTPTimeout())
        if out.status_code == 401:
            self._refreshToken()
            out = requests.delete(url,
                                  headers=self.config['headers'],
                                  verify=self.config['verify'],
                                  params=params, timeout=getHTTPTimeout())
        return out

    def _requestwrap(self, call_type, api_path, **kwargs):
        """"""
        self.__setdebug()
        ret = None
        params = {}
        if kwargs.get('query_params'):
            params = kwargs.get('query_params')
        retries = getHTTPRetries()
        while retries > 0:
            try:
                if call_type == "GET":
                    ret = self._get(api_path, params)
                elif call_type == "PUT":
                    ret = self._put(api_path, kwargs.get('body_params'), params)
                elif call_type == "POST":
                    if kwargs.get('body_params'):
                        ret = self._post(api_path, kwargs.get('body_params'), params)
                    else:
                        raise ValueError(
                            f"Missing the body parameter for POST to '{api_path}'")
                elif call_type == "DELETE":
                    ret = self._delete(api_path, params)
            except requests.exceptions.ReadTimeout as ex:
                self.logger.error(f"Got ReadTimeout exception: {ex}. Retrying up to {retries} times")
                retries -= 1
                continue
            break
        return ret


    def request(self, call_type, api_path, **kwargs):
        """Request Wrapper for SENSE-0 API (GET, PUT, POST, DELETE)"""
        if 'content_type' in kwargs or 'accept_type' in kwargs:
            self._setHeaders(content=kwargs.get('content_type', 'json'), accept=kwargs.get('accept_type', 'json'))

        ret = self._requestwrap(call_type, api_path, **kwargs)

        if ret is not None and ret.status_code >= 400 and ret.headers.get("content-type") == "application/json":
            json = ret.json()
            exc = ValueError(f"Returned code {ret.status_code} with error {json.get('exception')}. Full json return: {str(json)}")
            exc.json = json
            raise exc

        # If request headers and return headers are json, return json
        # In case of failure - return text. Reason for doing so is to
        # avoid issue of diff interpretation of json in java and python
        # e.g. Java API will return str inside quotes (which is valid json based on RFC),
        # but python (or atleast requests) will return str without quotes.
        try:
            if ret and self.config['headers'].get('Accept') == "application/json" and ret.headers.get("content-type") == "application/json":
                return ret.json()
            if ret and self.config['headers'].get('Accept') == "application/xml" and ret.headers.get("content-type") == "application/xml":
                return ret.text  # TODO: valudate XML in ret text
        except:
            pass
        return ret.text if ret.text is not None else ret



#!/usr/bin/env python3
# coding: utf-8
"""
    SENSE-SiteRM Frontend API
"""
from sense.client.siterm.requestwrapper import RequestWrapper


class FrontendApi:
    """Frontend API for SENSE-SiteRM"""
    def __init__(self):
        self.client = RequestWrapper()

    def _getSitename(self, **kwargs):
        sitename = kwargs.get("sitename", None)
        if not sitename:
            raise Exception("Sitename is required for get_configuration")
        return sitename

    def get_configuration(self, **kwargs):
        """Get configuration from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/frontend/configuration",
                                       **{"verb": "GET", "data": {}})

    def get_hostdata(self, **kwargs):
        """Get host data from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/frontend/gethosts",
                                       **{"verb": "GET", "data": {}})

    def get_switchdata(self, **kwargs):
        """Get switch data from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/frontend/getswitchdata",
                                       **{"verb": "GET", "data": {}})

    def get_topology(self, **kwargs):
        """Get topology from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/topo/gettopology",
                                       **{"verb": "GET", "data": {}})

    def get_activedeltas(self, **kwargs):
        """Get active deltas from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"/{sitename}/sitefe/json/frontend/getactivedeltas",
                                       **{"verb": "GET", "data": {}})

    def get_qosdata(self, **kwargs):
        """Get QoS data from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"{sitename}/sitefe/json/frontend/getqosdata",
                                       **{"verb": "GET", "data": {}})

    def get_metrics(self, **kwargs):
        """Get metrics from the SENSE-SiteRM API"""
        sitename = self._getSitename(**kwargs)
        return self.client.makeRequest(sitename=sitename,
                                       url=f"{sitename}/sitefe/json/frontend/metrics",
                                       **{"verb": "GET", "data": {}})
